local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local s = SpellFlashAddon
local x = s.UpdatedVariables

BittensSpellFlashLibrary2 = {}
local c = BittensSpellFlashLibrary2
local AddonName
local a

local Interruptible = {}

local HideHilighting = false
hooksecurefunc(
    "ActionButton_ShowOverlayGlow", 
    function(self)
        if HideHilighting and self.overlay then
            self.overlay:Hide()
            ActionButton_HideOverlayGlow(self)
        end
    end)
s.AddSettingsListener(function()
    HideHilighting = false
end)

c.SLOW_MELEE_DEBUFFS = {
    68055, -- Judgements of the Just 
    58180, -- Infected Wounds 
    59921, -- Frost Fever 
    6343, -- Thunder Clap 
    8042, -- Earth Shock 
    50285, -- Dust Cloud 
    16914, -- Hurricane 
    90314, --Tailspin
}

c.ARMOR_DEBUFFS = {
    7386, -- Sunder Armor
    8647, -- Expose Armor
    91565, -- Faerie Fire
    35387, -- Corrosive Spit
    50498, -- Tear Armor
}

c.PHYSICAL_DAMAGE_DEBUFFS = {
    702, -- Curse of Weakness
    99, -- Demoralizing Roar
    1160, -- Demoralizing Shout 
    81130, -- Scarlet Fever 
    26017, -- Vindication
}

c.BLEED_DEBUFFS = {
    33876, -- Mangle
    46857, -- Trauma
    16511, -- Hemorrhage
    50271, -- Tendon Rip
    35290, -- Gore
    57386, -- Stampede
}

c.FIVE_PERCENT_BUFFS = {
    1126, -- Mark of the Wild
    20217, -- Blessing of Kings
    90363, -- Embrace of the Shale Spider
}

c.STAMINA_BUFFS = {
    79105, -- Power Word: Fortitude
    90364, -- Qiraji Fortitude
    6307, -- Blood Pact
    469, -- Commanding Shout
}

c.SHADOW_RESISTANCE_BUFFS = {
    19891, -- Resistance Aura
    79107, -- Shadow Protection
}

c.INTELLECT_BUFFS = {
    79058, -- Arcane Brilliance
    79039, -- Dalaran Brilliance?
    79038, -- Dalaran Brilliance?
    52109, -- Flametongue Totem
    54424, -- Fel Intelligence
}

function c.Init(addonName, addonTable)
    AddonName = addonName
    a = addonTable
end

function c.Run(...)
    local ranOne = false
    for i=1,select("#", ...) do
        local runInfo = select(i, ...)
        
        local func = runInfo[1]
        local spec = runInfo[2]
        local offSwitch = runInfo[3]
        local form = runInfo[4]
        
        if s.TalentMastery(spec) 
            and not c.Selected(offSwitch) 
            and (form == nil or s.Form(form)) then
            
            func()
            ranOne = true
        end
    end
    HideHilighting = ranOne
end

-- remove in favor of a.GetConfig() when possible
function c.Selected(switch)
    return s.GetModuleConfig(AddonName, switch)
end

function c.RotateCooldowns(uncontrolledBuffs, ...)
    for tmp,id in pairs(uncontrolledBuffs) do
        if s.Buff(id, "player") then
            return
        end
    end
    for i=1,select("#", ...) do
        local buff = select(i, ...)
        local spell = a.spells[buff]
        if spell and spell.Buff then
            buff = spell.Buff
        end
        if c.HasBuff(buff, spell.NoGCD, spell.EarlyRefresh) then
            return
        end
    end
    return a.Flash(...)
end

function c.HasBuff(name, noGCD, durationRequired, noncombatDurationRequired)
    if noncombatDurationRequired and not s.InCombat() then
        durationRequired = noncombatDurationRequired
    elseif not durationRequired then
        durationRequired = .2
    end
    return c.GetBuffDuration(name, noGCD) > durationRequired
end

function c.GetBuffDuration(name, noGCD)
    return s.BuffDuration(c.GetID(name), "player") - c.GetBusyTime(noGCD)
end

function c.GetBuffStack(name)
    return s.BuffStack(c.GetID(name), "player")
end

function c.HasMyDebuff(name, noGCD, durationRequired)
    if not durationRequired then
        durationRequired = .2
    end
    return c.GetMyDebuffDuration(name, noGCD) > durationRequired
end

function c.GetMyDebuffDuration(name, noGCD)
    return s.MyDebuffDuration(c.GetID(name)) - c.GetBusyTime(noGCD)
end

function c.GetDebuffDuration(name, noGCD)
    return s.DebuffDuration(c.GetID(name)) - c.GetBusyTime(noGCD)
end

function c.SetInterrutpible(...)
    Interruptible = {}
    for i=1,select("#", ...) do
        Interruptible[s.SpellName(c.GetID(select(i, ...)), true)] = true
    end
end

function c.GetBusyTime(noGCD)
    local casting = s.CastingOrChanneling(nil, "player")
    if not casting or Interruptible[s.CastingOrChannelingName()] then
        casting = 0
    end
    local gcd = s.GlobalCooldown()
    if noGCD or casting > gcd then
        return casting
    else
        return gcd
    end
end

function c.GetCooldown(name)
    local cd = s.SpellCooldown(c.GetID(name)) - c.GetBusyTime()
    if cd < 0 then
        return 0
    else
        return cd
    end
end

function c.GetIDs(...)
    local ids = {}
    for i=1,select("#", ...) do
        local id = c.GetID(select(i, ...))
        if id then
            table.insert(ids, id)
        end
    end
    return ids
end

function c.GetID(name)
    local spell = a.spells[name]
    if spell and spell.ID then
        return spell.ID
    else
        print("No spell defined (or no ID attribute):", name)
    end
end

function c.SelfBuffNeeded(name)
    return not c.HasBuff(name, false, 0, 5 * 60)
end

function c.RaidBuffNeeded(idTable)
    local duration = 0
    if not s.InCombat() then
        duration = 5 * 60
    end
    local flags = "raid|all|range"
    if idTable == c.INTELLECT_BUFFS then
        flags = flags .. "|mana"
    end
    return not s.Buff(idTable, flags, duration)
end